package de.jottyfan.minecraft.quickiefabric.blocks;

import java.util.Arrays;
import java.util.List;

import com.mojang.serialization.MapCodec;

import de.jottyfan.minecraft.quickiefabric.items.QuickieItems;
import net.fabricmc.fabric.api.object.builder.v1.block.FabricBlockSettings;
import net.minecraft.block.AbstractBlock;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.Blocks;
import net.minecraft.block.CropBlock;
import net.minecraft.block.IceBlock;
import net.minecraft.block.ShapeContext;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.fluid.FluidState;
import net.minecraft.fluid.Fluids;
import net.minecraft.item.ItemConvertible;
import net.minecraft.item.ItemStack;
import net.minecraft.loot.context.LootContextParameterSet.Builder;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Hand;
import net.minecraft.util.ItemScatterer;
import net.minecraft.util.collection.DefaultedList;
import net.minecraft.util.hit.BlockHitResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.random.Random;
import net.minecraft.util.shape.VoxelShape;
import net.minecraft.world.BlockView;
import net.minecraft.world.World;

/**
 *
 * @author jotty
 *
 */
public class BlockSulforpad extends CropBlock {

	public BlockSulforpad(AbstractBlock.Settings settings) {
		super(settings.nonOpaque().blockVision(Blocks::never).solidBlock(Blocks::never));
	}

	protected BlockSulforpad() {
		super(FabricBlockSettings.create());
	}

	@Override
	public List<ItemStack> getDroppedStacks(BlockState state, Builder builder) {
		Integer age = this.getAge(state);
		Integer number = age < 3 ? 1 : (age < 6 ? 2 : 3);
		return Arrays.asList(new ItemStack[] { new ItemStack(QuickieItems.SULPHOR, number) });
	}

	@Override
	public VoxelShape getOutlineShape(BlockState state, BlockView world, BlockPos pos, ShapeContext context) {
		return Block.createCuboidShape(1.0, 0.0, 1.0, 15.0, 1.5, 15.0);
	}

	@Override
	protected boolean canPlantOnTop(BlockState floor, BlockView world, BlockPos pos) {
		Boolean lavaBase = floor.getBlock().equals(Blocks.LAVA) || floor.getBlock().equals(Blocks.LAVA_CAULDRON);
		return lavaBase && world.getBlockState(pos.up()).isAir();
	}

	@Override
	protected ItemConvertible getSeedsItem() {
		return QuickieItems.SULPHOR;
	}

	private void spawnHarvested(World world, BlockPos pos, BlockState state) {
		DefaultedList<ItemStack> list = DefaultedList.of();
		getDroppedStacks(state, null).forEach(itemStack -> {
			list.add(itemStack);
		});
		ItemScatterer.spawn(world, pos, list);
	}

	@Override
	public ActionResult onUse(BlockState state, World world, BlockPos pos, PlayerEntity player, Hand hand,
			BlockHitResult hitResult) {
		if (!world.isClient && isMature(state)) {
			spawnHarvested(world, pos, state);
			world.setBlockState(pos, state.with(AGE, 0));
		}
		return ActionResult.PASS;
	}

	@Override
	public MapCodec<? extends BlockSulforpad> getCodec() {
		return BlockSulforpad.createCodec(BlockSulforpad::new);
	}
}
